c       ccompPathEnd
c       compute path end information. including multiple scattering and mag.
c       deflection
c
      subroutine ccompPathEnd
      implicit none

#include  "Ztrack.h"
#include  "Ztrackv.h"
#include  "Ztrackp.h"
c
c
c         get endpoint coord. assuming no scat, no mag. effect. at first
c         MovedTrack has complete information at path end.
       MovedTrack = TrackBefMove   !   copy TrackBefMove into MovedTrack. 
       call cmoveStreight(IntInfArray(ProcessNo).length, 
     *             TrackBefMove.vec.w)
c         If a chargde ptcl, compute energy loss by dE/dx  and  reset energy
c         in MovedTrack. If it is too large, path is truncated. Also compute
c         deflection.
       if(IntInfArray(ProcessNo).length .gt. 0.0 ) then
          if(TrackBefMove.p.charge .ne. 0) then
c            if(.not. (mod(HowGeomag, 10) .eq. 1 .and.     this existed from uv4.92 to 5.13
c     *                Zfirst .eq. 0.)) then
             call cputEnergyLoss
c               endif
             if(MoveStat .ne. Dead) then
                call cputDeflection
             endif
          endif
       endif
       end
c     --------------------------------------------------------
       subroutine cmoveStreight(leng, dir)
c        move a track by leng m
c         after this is called,  MovedTrack has inf for moved pos.
c         MovedTrack.pos.where may not be correct. It must be
c         corrected by seeing if the track passes acrosss a
c         observation layer.
       implicit none

#include  "Ztrack.h"
#include  "Ztrackp.h"
#include  "Ztrackv.h"
#include  "Zearth.h"


       real*8  leng, cnewcos
       record /coord/ dir        ! input.
c
c            new coord in xyz system.

       MovedTrack.pos.xyz.r(1) = TrackBefMove.pos.xyz.r(1)
c     *                      + TrackBefMove.vec.w.r(1) * leng
     *                      + dir.r(1) * leng
       MovedTrack.pos.xyz.r(2) = TrackBefMove.pos.xyz.r(2)
c     *                      + TrackBefMove.vec.w.r(2) * leng
     *                      + dir.r(2) * leng
       MovedTrack.pos.xyz.r(3) = TrackBefMove.pos.xyz.r(3)
c     *                      + TrackBefMove.vec.w.r(3) * leng
     *                      + dir.r(3) * leng
       call csetPos(MovedTrack.pos) ! set height, radial length, thickness
c            get new cos
       MovedTrack.vec.coszenith = cnewcos(MovedTrack.pos.radiallen,
     *  TrackBefMove.vec.coszenith, leng)
       if(TimeStructure ) then
          call cgetBeta(MovedTrack.p, Beta)  
c             note righthand is not MovedTrack.t
          if(Beta .gt. 0.) then
             MovedTrack.t = TrackBefMove.t + leng/Beta
          endif
       endif
       end
c      -----------------------------------
       subroutine cputEnergyLoss
c         observation layer.
       implicit none

#include  "Zcode.h"
#include  "Ztrack.h"
#include  "Ztrackv.h"
#include  "Ztrackp.h"
 
c     
       real*8  rho, cvh2den, dedt, thick, leng, cupsilon
       save dedt
       real*8  csyncTELoss
       integer jcut
       real*8  dedx  ! output.  dedt is put


       real*8  dedxmu
c
c          first consider the synchrotron loss.
c
       if( TrackBefMove.p.fm.p(4) .gt. MagBremEmin) then
         if( MagBrem .eq. 1 .and. TrackBefMove.p.code .eq. kelec) then
	    Upsilon = cupsilon(TrackBefMove.p, Mag)
	    if(Upsilon .gt. UpsilonMin) then
c                 compute energy loss due to sychrotron rad.
                 EnergyLoss = csyncTELoss(Upsilon) * 
     *           IntInfArray(ProcessNo).length
                 if(Reverse .eq. 0) then
                    MovedTrack.p.fm.p(4) = 
     *                MovedTrack.p.fm.p(4) -  EnergyLoss
                 elseif(Reverse .eq. 2) then
                    MovedTrack.p.fm.p(4) = 
     *                MovedTrack.p.fm.p(4) +  EnergyLoss
                 endif
c                 don't  worry about death
	    endif
         endif
       endif 
c            
c           dE/dX
c
       rho = cvh2den(TrackBefMove.pos.height)
       call cdedxInAir(TrackBefMove.p, rho, dedt)  ! dedt; GeV/(kg/m2)
       if(TrackBefMove.p.code .eq. kmuon ) then
c           dE/dx due to muon pair, brem, nuc.i
          call cmudEdx(MuNI, MuBr, MuPr, TrackBefMove.p.fm.p(4),
     *         dedxmu)   ! dedxmu in GeV/(g/cm2)
          dedxmu = dedxmu /10.   !  GeV/(kg/m2)
          dedt = dedt + dedxmu
       endif

       EnergyLoss =  dedt * IntInfArray(ProcessNo).thickness

       if(Reverse .eq. 0) then
           MovedTrack.p.fm.p(4) = MovedTrack.p.fm.p(4) -  EnergyLoss          
c           see if <=mass
           if(MovedTrack.p.fm.p(4) .lt. MovedTrack.p.mass) then
              EnergyLoss =( TrackBefMove.p.fm.p(4) - MovedTrack.p.mass )
              if(dedt .gt. 0.) then
                 thick =EnergyLoss / dedt
              else
                 thick = 0.
              endif
              MovedTrack.p.fm.p(4) = MovedTrack.p.mass

c              call cthick2len(TrackBefMove.pos.height, 
c     *          TrackBefMove.vec.coszenith, thick, leng, thick, jcut)
c                   Mar.18
              call cthick2len(TrackBefMove, thick, leng, thick, jcut)
c               reset position information in MovedTrack
              call cmoveStreight(leng, TrackBefMove.vec.w)
              MoveStat = Truncated
           endif
c                reset 3 momenta px, py, pz 
c                 (but assume direction is unchanged)
           call ce2p(MovedTrack)
       elseif(Reverse .eq. 2) then
          MovedTrack.p.fm.p(4) = MovedTrack.p.fm.p(4) +  EnergyLoss
c                reset 3 momenta px, py, pz 
c                 (but assume direction is unchanged)
          call ce2p(MovedTrack)
c          MoveStat = Truncated
       else
c          MoveStat = Truncated
       endif 
       return
c      **************
       entry cqElossRate(dedx)
c      **************
c         inquire the dedt; which may be used when particle cross 
c         an obs. level and recompute the energy
       dedx = dedt
       end
c      --------------------------------------------------------------
       subroutine cputDeflection
       implicit none

#include  "Ztrack.h"
#include  "Ztrackp.h"
#include  "Ztrackv.h"
#include  "Zelemagp.h"

c
       real*8 dt, dispx, dispy
       logical nodef


       if(Reverse .eq. 0) then
          nodef = OneDim .ne. 0  .or.
     *    (mod(HowGeomag, 10) .eq. 1 .and. Zfirst.pos.depth .eq. 0.)
       else
          nodef = .false.
       endif
       if(.not. nodef) then
c
c         compute   magnetic deflection first. independently of scattering.
c         system is xyz
          call cmagDef
       endif
       if(.not. nodef .and. Reverse .eq. 0) then
          call celecDef(dispx, dispy)
       endif
       if(.not. nodef) then
          call csetPos(MovedTrack.pos)
          call cgetZenith(MovedTrack, MovedTrack.vec.coszenith)
c          reset momentum to be compatible with direction cos.
          call cresetMom(MovedTrack)
c
          if(TimeStructure .and. Reverse .eq. 0) then  ! only for mul. scat
c                compute excess path length to be added to streight path
             call cexcessLen(dispx, dispy, dt)
             if(Beta .gt. 0.) then
                MovedTrack.t = MovedTrack.t + dt/Beta
             endif
          endif
       endif
       end
c      *************************
#include "ZsubstRec.h"
       subroutine cmagDef
       implicit none
#include  "Zobs.h"
#include  "Zobsp.h"
#include  "Ztrack.h"
#include  "Ztrackp.h"
#include  "Ztrackv.h"
#include  "Zelemagp.h"

c

c              by  Geomag  (dr and ddirec)
       record /coord/ dispmr, dispmd
       record /magfield/ tempmag
       real*8  leng
       real*8 temp1, temp2
ccc    *   ,norm
       integer i

       integer icon
       record /coord/  middle
       logical usemiddle, high, usecmiddle
       real*8 cheight
       data cheight/30.d3/  
       save cheight

       leng = IntInfArray(ProcessNo).length
c
c            UseRungeKutta         Height       
c                0                  any         Mag and cmagneticDef

c                1                 >cheight     middle Mag and cmagneticDef
c                1                 <            Mag and cmagneticDef

c                2                 >            middle Mag and cmagneticDef 
c                                               or cbDefByRK2        
c                2                 <            Mag and cmagneticDef 

c                3                 >            middle Mag and cmagneticDef 
c                                               or cbDefByRK        
c                3                 <            Mag and cmagneticDef 

c                4                 >            use Mag at curved middle point
c                                               and estimate end point by 
c                                               cmagneticDef or cbDefByRK2
c                4                 <            Mag and cmagneticDef   

c                5                 >            use Mag at curved middle point
c                                               and estimate end point by 
c                                               cmagneticDef or cbDefByRK
c                5                 <            Mag and cmagneticDef   

c                6                 >            cbDefByRK2
c                6                 <            Mag and cmagneticDef 

c                7                 >            cbDefByRK
c                7                 <            Mag and cmagneticDef 
c
c                8      at any height           cbDefUser ; interface is
c                                               same as cbDefByRK
c
       if(UseRungeKutta .eq. 8 ) then
          call cbDefUser(TrackBefMove,  leng,  dispmr, dispmd)
#if defined SUBSTREC             
             MovedTrack.pos.xyz = dispmr ! this is not a dispalcement vector
#else 
             call csubstcoord(dispmr, MovedTrack.pos.xyz)
#endif
          goto 100
       endif
       high = TrackBefMove.pos.height .gt. cheight
       usemiddle = ( UseRungeKutta .ge. 1 .and.
     *               UseRungeKutta .le. 3 .and.
     *               high )
       usecmiddle = ( UseRungeKutta .ge. 4 .and.
     *               UseRungeKutta .le. 5 .and.
     *               high )


       if(usemiddle) then
          do i = 1, 3
             middle.r(i) = TrackBefMove.pos.xyz.r(i)+ 
     *          leng/2 * TrackBefMove.vec.w.r(i)
          enddo
          middle.sys='xyz'

          call cgeomag(YearOfGeomag,  middle,
     *                tempmag, icon)
          call ctransMagTo('xyz', middle,
     *        tempmag, tempmag)
       elseif(usecmiddle) then
c            get curved middle point
          call cmagneticDef(TrackBefMove, Mag, leng/2.0d0,
     *     dispmr, dispmd)  
          do i = 1, 3
             middle.r(i) = TrackBefMove.pos.xyz.r(i) + dispmr.r(i)
          enddo
          middle.sys ='xyz'
          call cgeomag(YearOfGeomag,  middle,
     *                tempmag, icon)
          call ctransMagTo('xyz', middle,
     *        tempmag, tempmag)
       endif

       if(usemiddle .and. UseRungeKutta .eq. 1) then
          Mag = tempmag
       elseif( usemiddle .and.  UseRungeKutta .eq. 2 ) then
          temp1 = tempmag.x**2 + tempmag.y**2 + tempmag.z**2
          temp2 = Mag.x**2 + Mag.y**2 + Mag.z**2
c           if  dB/B~ dB^2/B^2/2 > 0.4 %, use RungeKutta.
          if( abs(temp1-temp2)/temp1/2.0 .gt. 4.0d-3) then
             call cbDefByRK2(TrackBefMove,  leng, dispmr, dispmd)
#if defined SUBSTREC             
             MovedTrack.pos.xyz = dispmr ! this is not a dispalcement vector
#else 
             call csubstcoord(dispmr, MovedTrack.pos.xyz)
#endif
             goto 100
          else
             Mag = tempmag
          endif
       elseif(usemiddle .and. UseRungeKutta .eq. 3) then
          temp1 = tempmag.x**2 + tempmag.y**2 + tempmag.z**2
          temp2 = Mag.x**2 + Mag.y**2 + Mag.z**2
c           if  dB/B~ dB^2/B^2/2 > 0.45 %, use RungeKutta.
c                  next 4.5d-3 is very sensitive to cpu time
c                 if it was 4.0d-3, cpu time becomes twice.
          if( abs(temp1-temp2)/temp1/2.0 .gt. 4.5d-3) then
             call cbDefByRK(TrackBefMove,  leng, dispmr, dispmd)
#if defined SUBSTREC             
             MovedTrack.pos.xyz = dispmr ! this is not a dispalcement vector
#else 
             call csubstcoord(dispmr, MovedTrack.pos.xyz)
#endif
             goto 100
          else
             Mag = tempmag
          endif
       elseif(usecmiddle .and. UseRungeKutta .eq. 4) then
          temp1 = tempmag.x**2 + tempmag.y**2 + tempmag.z**2
          temp2 = Mag.x**2 + Mag.y**2 + Mag.z**2
c           if  dB/B~ dB^2/B^2/2 > 0.1 %, use RungeKutta.
          if( abs(temp1-temp2)/temp1/2.0 .gt. 4.0d-3) then
             call cbDefByRK2(TrackBefMove,  leng, dispmr, dispmd)
#if defined SUBSTREC             
             MovedTrack.pos.xyz = dispmr ! this is not a dispalcement vector
#else 
             call csubstcoord(dispmr, MovedTrack.pos.xyz)
#endif
             goto 100
          else
             Mag = tempmag
          endif
       elseif(usecmiddle .and. UseRungeKutta .eq. 5) then
          temp1 = tempmag.x**2 + tempmag.y**2 + tempmag.z**2
          temp2 = Mag.x**2 + Mag.y**2 + Mag.z**2
c           if  dB/B~ dB^2/B^2/2 > 0.45 %, use RungeKutta.
          if( abs(temp1-temp2)/temp1/2.0 .gt. 4.5d-3) then
             call cbDefByRK(TrackBefMove,  leng, dispmr, dispmd)
#if defined SUBSTREC             
             MovedTrack.pos.xyz = dispmr ! this is not a dispalcement vector
#else 
             call csubstcoord(dispmr, MovedTrack.pos.xyz)
#endif
             goto 100
          else
             Mag = tempmag
          endif
       endif

       if(UseRungeKutta .le. 5 .or. .not. high ) then
          call cmagneticDef(TrackBefMove, Mag, leng,
     *     dispmr, dispmd)

          do i = 1,  3
             MovedTrack.pos.xyz.r(i) = TrackBefMove.pos.xyz.r(i) +
     *         dispmr.r(i)
          enddo
       elseif(UseRungeKutta .eq. 6) then
          call cbDefByRK2(TrackBefMove,  leng, dispmr, dispmd)
#if defined SUBSTREC
          MovedTrack.pos.xyz = dispmr  ! this is not a dispalcement vector
#else 
          call csubstcoord(dispmr, MovedTrack.pos.xyz)
#endif
                                      !  but the vector itself.
       else
          call cbDefByRK(TrackBefMove,  leng, dispmr, dispmd)
#if defined SUBSTREC
          MovedTrack.pos.xyz = dispmr  ! this is not a dispalcement vector
                                      !  but the vector itself.
#else 
          call csubstcoord(dispmr, MovedTrack.pos.xyz)
#endif
       endif
 100   continue
#if defined SUBSTREC
       MovedTrack.vec.w = dispmd
#else
       call csubstcoord(dispmd, MovedTrack.vec.w)
#endif
       end
c        for IBM AIX
      subroutine csubstcoord(c1, c2)
      implicit none
#include "Zcoord.h"
      record /coord/ c1, c2   
      c2 = c1
      end

c      ==============================================================
       subroutine celecDef(dispx, dispy)
       implicit none

#include  "Ztrack.h"
#include  "Ztrackp.h"
#include  "Ztrackv.h"
#include  "Zelemagp.h"

       real*8 dispx, dispy  ! output. scatttering displacement
c
       record /coord/ dircos
c              by  Multiple Scattering

       real*8 tetax, tetay

       real*8 leng

       integer i

       leng = IntInfArray(ProcessNo).length

c           Multiples csattering. around MovedTrack
       call cmulScat(dispx, dispy, tetax, tetay)
c
c         form direction cos. if tx, ty are large, not correct.
       call cvec2dcos(tetax, tetay, dircos)
c
c            new direction of moved particle in xyz system. 
       call ctransVectZ(MovedTrack.vec.w,   dircos, 
     *           MovedTrack.vec.w)
c     +++++++++++++++++++++++++++++++++++++++++++++++++++++++
c               displacement vector to direction cos
       call cdispVec2Dcos(dispx, dispy, leng, dircos)
c               in xyz system
       call ctransVectZ(MovedTrack.vec.w, dircos, dircos)

       do i = 1, 3
          MovedTrack.pos.xyz.r(i) = MovedTrack.pos.xyz.r(i)  +
     *         leng * dircos.r(i)
       enddo
       end

